@@ -90,16 +90,13 @@
 #define LOG_LINE_MAXSIZE 65535
 
 
-#define LOGFILE_DELETION_CLASS "Logfile deletion"
+#define LOGFILE_RENAME_CLASS   "Log file rename"
+#define LOGFILE_DELETION_CLASS "Log file deletion"
 #define LOGFILE_DELETION_IMPACT "An attacker might have erased the logfile,"               \
                                 "or a log rotation program may have rotated the logfile."
 
-#define LOGFILE_DELETION_IMPACT_HIGH "An attacker seems to have erased the logfile, "      \
-                                     "and the change doesn't seem to be related to a log " \
-                                     "rotation program."
-
-#define LOGFILE_MODIFICATION_CLASS "Logfile inconsistency"
-#define LOGFILE_MODIFICATION_IMPACT "An attacker might have modified the logfile in order " \
+#define LOGFILE_MODIFICATION_CLASS "Log file inconsistency"
+#define LOGFILE_MODIFICATION_IMPACT "An attacker might have modified the log file in order " \
                                     "to remove the trace he left."
 
 
@@ -111,6 +108,7 @@ typedef struct {
 
         FILE *fd;
         FILE *metadata_fd;
+        prelude_bool_t need_position;
 
         off_t last_rotation_size;
         time_t last_rotation_time;
@@ -144,7 +142,8 @@ static FAMConnection fc;
 
 static int batch_mode = 0;
 static int fam_initialized = 0;
-static int ignore_metadata = 0;
+static file_server_metadata_flags_t metadata_flags = FILE_SERVER_METADATA_FLAGS_LAST;
+
 extern lml_config_t config;
 
 
@@ -295,9 +294,6 @@ static int file_metadata_read(monitor_fd_t *monitor, off_t *start, char **sumlin
         int line = 0, ret;
         char *offptr, *buf;
 
-        if ( ignore_metadata )
-                return 0;
-
         rewind(monitor->metadata_fd);
 
         *start = 0;
@@ -335,7 +331,7 @@ static int file_metadata_save(monitor_fd_t *monitor, off_t offset)
         int len, ret;
         char buf[METADATA_MAXSIZE];
 
-        if ( ignore_metadata )
+        if ( metadata_flags & FILE_SERVER_METADATA_FLAGS_NO_WRITE )
                 return 0;
 
         len = snprintf(buf, sizeof(buf), "%" PRELUDE_PRIu64 ":%s\n", offset, prelude_string_get_string(monitor->buf));
@@ -369,9 +365,6 @@ static int file_metadata_get_position(monitor_fd_t *monitor)
         int ret, have_metadata = 0;
         char buf[1024], sumline[METADATA_MAXSIZE], *sumptr;
 
-        if ( ignore_metadata )
-                return 0;
-
         sumptr = sumline;
         filename = lml_log_source_get_name(monitor->source);
 
@@ -389,26 +382,28 @@ static int file_metadata_get_position(monitor_fd_t *monitor)
         monitor->last_mtime = st.st_mtime;
 
         if ( ! have_metadata ) {
-                prelude_log(PRELUDE_LOG_INFO, "%s: No metadata available.\n", filename);
-                return fseek(monitor->fd, st.st_size, SEEK_SET);;
+                prelude_log(PRELUDE_LOG_INFO, "%s: No metadata available, starting from tail.\n", filename);
+                return fseek(monitor->fd, st.st_size, SEEK_SET);
         }
 
         if ( st.st_size < offset ) {
-                prelude_log(PRELUDE_LOG_INFO, "%s: Metadata available, but logfile got rotated, starting at 0.\n", filename);
+                prelude_log(PRELUDE_LOG_INFO, "%s: log file was rotated, starting from head.\n", filename);
                 monitor->last_size = 0;
                 return 0;
         }
 
         ret = fseek(monitor->fd, offset, SEEK_SET);
         if ( ret < 0 ) {
-                prelude_log(PRELUDE_LOG_ERR, "%s: could not seek to byte %" PRELUDE_PRIu64 ": %s.\n",
+                prelude_log(PRELUDE_LOG_ERR, "%s: error seeking to offset %" PRELUDE_PRIu64 ": %s.\n",
                             filename, offset, strerror(errno));
                 return -1;
         }
 
+        /*
+         * If the metadata checksum does not match, the file was probably rotated.
+         */
         if ( ! fgets(buf, sizeof(buf), monitor->fd) || strcmp(buf, sumptr) != 0 ) {
-                prelude_log(PRELUDE_LOG_WARN, "%s: Metadata available, but checksum is invalid, starting at 0.\n", filename);
-                logfile_modified_alert(monitor, &st);
+                prelude_log(PRELUDE_LOG_INFO, "%s: log file was rotated, starting from head.\n", filename);
                 monitor->last_size = 0;
                 return fseek(monitor->fd, 0, SEEK_SET);
         }
@@ -416,7 +411,7 @@ static int file_metadata_get_position(monitor_fd_t *monitor)
         monitor->last_size = offset;
         monitor->last_size += strlen(sumptr);
 
-        prelude_log(PRELUDE_LOG_INFO, "%s: Metadata available, starting log analyzis at offset %" PRELUDE_PRIu64 ".\n",
+        prelude_log(PRELUDE_LOG_INFO, "%s: resuming log analyzis at offset %" PRELUDE_PRIu64 ".\n",
                     filename, monitor->last_size);
 
         return 0;
@@ -428,10 +423,7 @@ static int file_metadata_get_position(monitor_fd_t *monitor)
 static int file_metadata_open(monitor_fd_t *monitor)
 {
         int fd;
-        char file[FILENAME_MAX], path[FILENAME_MAX], *ptr;
-
-        if ( ignore_metadata )
-                return 0;
+        char file[PATH_MAX], path[PATH_MAX], *ptr;
 
         strncpy(file, lml_log_source_get_name(monitor->source), sizeof(file));
 
@@ -441,7 +433,7 @@ static int file_metadata_open(monitor_fd_t *monitor)
         snprintf(path, sizeof(path), "%s/%s", METADATA_DIR, file);
 
         fd = open(path, O_CREAT|O_RDWR, S_IRUSR|S_IWUSR);
-        if ( fd < 0 && errno != EEXIST ) {
+        if ( fd < 0 ) {
                 prelude_log(PRELUDE_LOG_ERR, "error creating %s: %s.\n", path, strerror(errno));
                 return -1;
         }
@@ -451,6 +443,7 @@ static int file_metadata_open(monitor_fd_t *monitor)
         monitor->metadata_fd = fdopen(fd, "r+");
         if ( ! monitor->metadata_fd ) {
                 prelude_log(PRELUDE_LOG_ERR, "fdopen failed: %s.\n", strerror(errno));
+                close(fd);
                 return -1;
         }
 
@@ -520,11 +513,6 @@ static int check_logfile_data(monitor_fd_t *monitor, struct stat *st)
         if ( monitor->fd != stdin && ! monitor->need_more_read && st->st_size == monitor->last_size )
                 return 0;
 
-        if ( st->st_size < monitor->last_size ) {
-                monitor->last_size = 0;
-                rewind(monitor->fd);
-        }
-
         len = (st->st_size - monitor->last_size) + monitor->need_more_read;
         monitor->last_size = st->st_size;
 
@@ -580,6 +568,7 @@ static monitor_fd_t *monitor_new(lml_log_source_t *ls)
         }
 
         new->source = ls;
+        new->need_position = TRUE;
 
         ret = file_metadata_open(new);
         if ( ret < 0 ) {
@@ -618,6 +607,35 @@ static void monitor_destroy(monitor_fd_t *monitor)
 
 
 
+static int monitor_set_position(monitor_fd_t *monitor, const char *filename, int fd)
+{
+        int ret = 0;
+
+        if ( ! monitor->need_position )
+                return 0;
+
+        monitor->need_position = FALSE;
+
+        if ( metadata_flags & FILE_SERVER_METADATA_FLAGS_LAST )
+                return file_metadata_get_position(monitor);
+
+        else if ( metadata_flags & FILE_SERVER_METADATA_FLAGS_HEAD )
+                ret = fseek(monitor->fd, 0, SEEK_SET);
+
+        else if ( metadata_flags & FILE_SERVER_METADATA_FLAGS_TAIL )
+                ret = fseek(monitor->fd, 0, SEEK_END);
+
+        if ( ret < 0 ) {
+                prelude_log(PRELUDE_LOG_ERR, "%s: error seeking to the %s of the file: %s.\n",
+                            filename, (metadata_flags & FILE_SERVER_METADATA_FLAGS_TAIL) ? "tail" : "head",
+                            strerror(errno));
+                return -1;
+        }
+
+        return ret;
+}
+
+
 static int monitor_open(monitor_fd_t *monitor)
 {
         int ret, fd;
@@ -635,7 +653,7 @@ static int monitor_open(monitor_fd_t *monitor)
                 fd = fileno(monitor->fd);
                 fcntl(fd, F_SETFD, fcntl(fd, F_GETFD) | FD_CLOEXEC);
 
-                ret = file_metadata_get_position(monitor);
+                ret = monitor_set_position(monitor, filename, fd);
                 if ( ret < 0 )
                         return -1;
         }
@@ -662,6 +680,8 @@ static void monitor_close(monitor_fd_t *monitor)
 
         fclose(monitor->fd);
         monitor->fd = NULL;
+        monitor->last_size = 0;
+        monitor->last_mtime = 0;
 
         prelude_list_del(&monitor->list);
         prelude_list_add_tail(&inactive_fd_list, &monitor->list);
@@ -688,6 +708,7 @@ static void try_reopening_inactive_monitor(void)
  */
 static void check_modification_time(monitor_fd_t *monitor, struct stat *st)
 {
+        int ret;
         time_t old_mtime = monitor->last_mtime;
 
         monitor->last_mtime = st->st_mtime;
@@ -696,6 +717,22 @@ static void check_modification_time(monitor_fd_t *monitor, struct stat *st)
                 return; /* everythings sound okay */
 
         logfile_modified_alert(monitor, st);
+
+        /*
+         * If the logfile has been modified, we reposition the current
+         * descriptor to EOF, and start analyzing from this place.
+         */
+        if ( st->st_size < monitor->last_size ) {
+                ret = fseek(monitor->fd, 0, SEEK_END);
+                if ( ret < 0 ) {
+                        prelude_log(PRELUDE_LOG_ERR, "%s: error seeking to end of file: %s.\n",
+                                    lml_log_source_get_name(monitor->source), strerror(errno));
+                        return;
+                }
+
+                monitor->need_more_read = 0;
+                monitor->last_size = st->st_size;
+        }
 }
 
 
@@ -745,7 +782,7 @@ static int is_file_already_used(monitor_fd_t *monitor, struct stat *st)
         char buf[1024];
         int toff, soff;
         struct stat st_new;
-        const char *filename;
+        const char *filename, *ctxt;
         idmef_impact_t *impact;
         idmef_classification_t *classification;
         prelude_string_t *str;
@@ -763,14 +800,16 @@ static int is_file_already_used(monitor_fd_t *monitor, struct stat *st)
                         if ( st->st_ino == st_new.st_ino )
                                 return 0;
 
-                        fclose(monitor->fd);
+                        monitor_close(monitor);
                         monitor_open(monitor);
                 }
                 else monitor_close(monitor);
-                prelude_log(PRELUDE_LOG_INFO, "logfile %s has been renamed.\n", filename);
+                prelude_log(PRELUDE_LOG_INFO, "log file %s has been renamed.\n", filename);
+                ctxt = LOGFILE_RENAME_CLASS;
         } else {
-                prelude_log(PRELUDE_LOG_INFO, "logfile %s reached 0 hard link.\n", filename);
+                prelude_log(PRELUDE_LOG_INFO, "log file %s has been deleted.\n", filename);
                 monitor_close(monitor);
+                ctxt = LOGFILE_DELETION_CLASS;
         }
 
         ret = idmef_classification_new(&classification);
@@ -789,7 +828,7 @@ static int is_file_already_used(monitor_fd_t *monitor, struct stat *st)
                 idmef_classification_destroy(classification);
                 return -1;
         }
-        prelude_string_set_constant(str, LOGFILE_DELETION_CLASS);
+        prelude_string_set_ref(str, ctxt);
 
         idmef_impact_set_type(impact, IDMEF_IMPACT_TYPE_FILE);
         idmef_impact_set_completion(impact, IDMEF_IMPACT_COMPLETION_SUCCEEDED);
@@ -817,7 +856,33 @@ static int is_file_already_used(monitor_fd_t *monitor, struct stat *st)
         return -1;
 }
 
+static int process_file_event(monitor_fd_t *monitor)
+{
+        int ret;
+        struct stat st;
 
+        ret = fstat(fileno(monitor->fd), &st);
+        if ( ret < 0 ) {
+                prelude_log(PRELUDE_LOG_ERR, "couldn't fstat '%s'.\n", lml_log_source_get_name(monitor->source));
+                return -1;
+        }
+
+        if ( monitor->fd != stdin ) {
+                ret = is_file_already_used(monitor, &st);
+                if ( ret < 0 )
+                        return -1;
+        }
+
+        /*
+         * check mtime consistency.
+         */
+        check_modification_time(monitor, &st);
+
+        /*
+         * read and analyze available data.
+         */
+        return check_logfile_data(monitor, &st);
+}
 
 
 #ifdef HAVE_FAM
@@ -920,7 +985,7 @@ static int initialize_fam(void)
 
         ret = FAMOpen(&fc);
         if ( ret < 0 ) {
-                prelude_log(PRELUDE_LOG_WARN, "error initializing FAM: %s.\n", FamErrlist[FAMErrno]);
+                prelude_log(PRELUDE_LOG_WARN, "error initializing FAM: %s.\n", FAMErrno ? FamErrlist[FAMErrno] : "unknown");
                 return -1;
         }
 
@@ -964,11 +1029,9 @@ static int fam_setup_monitor(monitor_fd_t *monitor)
 }
 
 
-
 static int fam_process_event(FAMEvent *event)
 {
         int ret = 0;
-        struct stat st;
         monitor_fd_t *monitor = event->userdata;
 
         switch (event->code) {
@@ -978,21 +1041,7 @@ static int fam_process_event(FAMEvent *event)
                         return -1;
 
         case FAMDeleted:
-                ret = fstat(fileno(monitor->fd), &st);
-                if ( ret < 0 )
-                        prelude_log(PRELUDE_LOG_ERR, "fstat returned an error.\n");
-
-                /*
-                 * check mtime consistency.
-                 */
-                check_modification_time(monitor, &st);
-
-                /*
-                 * read and analyze available data.
-                 */
-                check_logfile_data(monitor, &st);
-
-                ret = is_file_already_used(monitor, &st);
+                ret = process_file_event(monitor);
                 break;
 
         case FAMExists:
@@ -1033,37 +1082,6 @@ static int fam_process_queued_events(void)
 
 
 
-static int process_file_event(monitor_fd_t *monitor)
-{
-        int ret;
-        struct stat st;
-
-        ret = fstat(fileno(monitor->fd), &st);
-        if ( ret < 0 ) {
-                prelude_log(PRELUDE_LOG_ERR, "couldn't fstat '%s'.\n", lml_log_source_get_name(monitor->source));
-                return -1;
-        }
-
-        if ( monitor->fd != stdin ) {
-                ret = is_file_already_used(monitor, &st);
-                if ( ret < 0 )
-                        return -1;
-        }
-
-        /*
-         * check mtime consistency.
-         */
-        check_modification_time(monitor, &st);
-
-        /*
-         * read and analyze available data.
-         */
-        return check_logfile_data(monitor, &st);
-}
-
-
-
-
 int file_server_monitor_file(lml_log_source_t *ls)
 {
         monitor_fd_t *new;
@@ -1173,9 +1191,9 @@ void file_server_start_monitoring(void)
 }
 
 
-void file_server_set_ignore_metadata(void)
+void file_server_set_metadata_flags(file_server_metadata_flags_t flags)
 {
-        ignore_metadata = 1;
+        metadata_flags = flags;
 }
 
 
